/************************************************/
/*						*/
/*	definition file for Martin Eyre		*/
/*	hardware V1.3				*/
/*  add support for Nintendo flash cart         */
/*						*/
/************************************************/

unsigned ctrl_port;
unsigned data_port;
unsigned status_port;

#define CTRL    ctrl_port
#define DATA    data_port
#define STATUS  status_port

/*
 * Bits for MUX (C B A):
 *      ~AUTOFDXT ~SCLINT ~STROBE
 * Bits in CTRL register (7..0):
 *      N/A N/A N/A IRQEnable ~SCLINT INIT ~AUTOFDXT ~STROBE
 */

#define MASK            0x00
#define nSTROBE         0x01
#define nAUTOFDXT       0x02
#define INIT            0x04
#define nSCLINT         0x08

/* CTRL MUX */
#define GET_LOW         MASK |           nSCLINT   | nSTROBE	// 000 0
#define GET_HIGH        MASK |           nSCLINT	// 001 1
#define SET_DATA        MASK |                       nSTROBE	// 010 2
#define DATA_REG        MASK	// 011 3
#define LOW_ADDR        MASK | INIT    | nSCLINT   | nSTROBE	// 100 4
#define HIGH_ADDR       MASK | INIT    | nSCLINT	// 101 5
#define READ_ROM        MASK | INIT    |             nSTROBE	// 110 6
#define NOP             MASK | INIT	// 111 7

/* Load a register: toggle CK (OFF and ON) */
#define LOAD(reg)       OUTPORT(CTRL, reg);DELAY;  OUTPORT(CTRL, NOP)

/* Put data on the bus */
#define WRITE(val)      OUTPORT(DATA, val)

/* disable LED */
#define LED_OFF         WRITE(0x00); OUTPORT(CTRL, SET_DATA)

/*
 * Bits read (7..0):
 *      BUSY ~ACKPR PE SLCT ERROR N/A N/A N/A
 * Bits from REGISTER (D7..D4, D3..D0):
 *      SLCT PE BUSY ~ACK
 */
#define D7(val)             (val & 0x10) << 3
#define D6(val)             (val & 0x20) << 1
#define D5(val)             ((val ^ 0x80) & 0x80) >> 2
#define D4(val)             (val & 0x40) >> 2
#define D3(val)             (val & 0x10) >> 1
#define D2(val)             (val & 0x20) >> 3
#define D1(val)             ((val ^ 0x80) & 0x80) >> 6
#define D0(val)             (val & 0x40) >> 6

void SET_ROM_ON(void)
{
  WRITE(0x00);
  OUTPORT(CTRL, READ_ROM);
  OUTPORT(CTRL, NOP);
}

void SET_ROM_OFF(void)
{
  WRITE(0x01);
  OUTPORT(CTRL, READ_ROM);
  OUTPORT(CTRL, NOP);
}

/* read data from RAM(1) or ROM(0) */
BYTE READ(BYTE Mode)
{
  BYTE low, high;
  if (Mode == 1)
    SET_ROM_ON();
  OUTPORT(CTRL, GET_LOW);
  low = INPORT(STATUS);
  OUTPORT(CTRL, GET_HIGH);
  high = INPORT(STATUS);
  if (Mode == 1)
    SET_ROM_OFF();
  return (D7(high) | D6(high) | D5(high) | D4(high) | D3(low) | D2(low) |
	  D1(low) | D0(low));
}

/* write data in bank-switch IC */
void WRITE_BKSW(data, address)
{
  WRITE((BYTE) data);
  LOAD(DATA_REG);
  WRITE((BYTE) (address >> 8));
  LOAD(HIGH_ADDR);
  WRITE((BYTE) (address & 0x00FF));
  LOAD(LOW_ADDR);
  WRITE(0x00);
  OUTPORT(CTRL, SET_DATA);
  WRITE(0x01);
  WRITE(0x00);
  OUTPORT(CTRL, NOP);
}

/* write data in RAM */
void WRITE_MEM(val)
{
  WRITE((BYTE) val);
  LOAD(DATA_REG);
  WRITE(0x00);
  OUTPORT(CTRL, SET_DATA);
  SET_ROM_ON();
  OUTPORT(CTRL, SET_DATA);
  SET_ROM_OFF();
//    OUTPORT(CTRL, SET_DATA | MREQ);
  WRITE(0x01);
  WRITE(0x00);
  OUTPORT(CTRL, NOP);
}

/* write data in Flash IC */
void WRITE_FLASH(val)
{
  WRITE((BYTE) val);
  LOAD(DATA_REG);
  WRITE(0x00);
  OUTPORT(CTRL, SET_DATA);
  WRITE(0x02);
  WRITE(0x00);
  OUTPORT(CTRL, NOP);
}

/* write data in Nintendo flash cart */
void WRITE_NFLASH(val)
{
  WRITE((BYTE) val);
  LOAD(DATA_REG);
  WRITE(0x00);
  OUTPORT(CTRL, SET_DATA);
  WRITE(0x01);
  WRITE(0x00);
  OUTPORT(CTRL, NOP);
}

/* write low address byte */
void ADDRESS_LOW(val)
{
  WRITE((BYTE) val);
  LOAD(LOW_ADDR);
}

/* write high address byte */
void ADDRESS_HIGH(val)
{
  WRITE((BYTE) val);
  LOAD(HIGH_ADDR);
}

/* usage of hardware */
void usage_hardware(void)
{
  printf("\t-l\tSpecify the port to use (default is 1 = LPT1)\n");
}

/* init port */
void init_port(int port)
{
#ifdef LINUX
  //I like this more
  data_port = printer_port[port];
#else
  data_port = *(unsigned far *) MK_FP(0x0040, 6 + 2 * port);
#endif
  if (data_port == 0) {
    printf("Can't find address of parallel port %d...\n", port);
    exit(1);
  } else {
    getPermissions(data_port, 3);
    status_port = data_port + 1;
    ctrl_port = data_port + 2;
    if (data_port != 0x378)
      printf("Parallel port %d is located at %X-%X\n", port, data_port,
	     ctrl_port);
  }
}

/* testprogram for CARTIO and C3 */
void test()
{
  printf("\nD0 - D7 data lines check !\n");
  printf("Check pins <22 up to 29> of GB connector\n");
  WRITE(0x00);
  LOAD(LOW_ADDR);
  LOAD(HIGH_ADDR);
  LOAD(DATA_REG);
  OUTPORT(CTRL, SET_DATA);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  WRITE(0xFF);
  LOAD(DATA_REG);
  OUTPORT(CTRL, SET_DATA);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nA0 - A7 address lines check !\n");
  printf("Check pins <6 up to 13> of GB connector\n");
  WRITE(0x00);
  LOAD(LOW_ADDR);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  WRITE(0xFF);
  LOAD(LOW_ADDR);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nA8 - A15 address lines check !\n");
  printf("Check pins <14 up to 21> of GB connector\n");
  WRITE(0x00);
  LOAD(HIGH_ADDR);
  printf("They should all be low. Press <RETURN>\n");
  getch();

  WRITE(0xFF);
  LOAD(HIGH_ADDR);
  printf("Now, they should all be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nCS line check !\n");
  printf("check pin <5> of GB connector\n");
  SET_ROM_OFF();
//        OUTPORT(CTRL, MREQ);
  printf("Should be low. Press <RETURN>\n");
  getch();
  SET_ROM_ON();

//        OUTPORT(CTRL, NOP);
  printf("Should be high. Press <RETURN>\n");
  getch();
  SET_ROM_OFF();

  printf("\n\n\nRD line check !\n");
  printf("check pin <4> of GB connector\n");
  OUTPORT(CTRL, NOP);
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(CTRL, SET_DATA);
  printf("Should be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nWR line check !\n");
  printf("check pin <3> of GB connector\n");
  WRITE(0x01);
  LOAD(DATA_REG);
  OUTPORT(CTRL, SET_DATA);
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(CTRL, NOP);
  printf("Should be high. Press <RETURN>\n");
  getch();

  printf("\n\n\nAudio IN line check !\n");
  printf("check pin <31> of GB connector\n");
  WRITE(0x02);
  LOAD(DATA_REG);
  OUTPORT(CTRL, SET_DATA);
  printf("Should be low. Press <RETURN>\n");
  getch();

  OUTPORT(CTRL, NOP);
  printf("Should be high. Press <RETURN>\n");
  getch();

  LED_OFF;
  printf("\n\n\nTest complete !\n\n");
}
